from contextlib import contextmanager


class Signal(object):
    """
    Creates a new signal instance.
    """

    def __init__(self):
        self.listeners = []

    def register(self, function):
        """
        Registers a function or callable to be called
        when the signal is triggered. This method can
        also be used as a decorator.

        :param function: The callable.
        """
        self.listeners.append(function)
        return function

    def trigger(self, *args, **kwargs):
        """
        Trigger the signals with positional arguments
        *args* and keyword arguments *kwargs*.

        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        """
        stack = []
        for item in self.listeners:
            stack.append((item, item(*args, **kwargs)))
        return stack

    def silence(self, *functions):
        """
        Silences the signal and removes the specified
        *functions* from the internal listeners list.

        :param functions: Positional argument of the
            functions to remove.
        """
        for item in functions:
            self.listeners.remove(item)

    def clear(self):
        """
        Clears the signal and removes any listeners.
        """
        self.listeners = []

    @contextmanager
    def connected_to(self, function):
        """
        Temporarily register one function while the
        block is still active, and then remove it
        from the signal.

        :param function: The function to register.
        """
        self.register(function)
        try:
            yield
        finally:
            self.silence(function)

    def __contains__(self, function):
        """
        Check if the *function* is registered on the
        signal as a listener.

        :param function: The function to check.
        """
        return function in self.listeners
